/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: acse.c
 *
 * Purpose: The Association Control Service Element functions which takes
 *          care of association and disconnection between the client and
 *          IR kernel.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *                 ACSE()
 *                 A_associate()
 *                 A_release()
 *                 A_abort
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Log: ir.h,v $
 *
 *****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "ir.h"
#include "irstatab.h"
#include "irerror.h"
#include "irconst.h"
#include "irassoc.h"
#include "cspabort.h"
#include "cspconn.h"

/*****************************************************************************
 *
 * Function: ACSE
 *
 * Purpose: The Association Control Service Element manages the associations
 *          (A_associate()) and disconnection (A_release() and A_abort()).
 *
 * Input Arguments:
 *          assoc_list - pointer to the association list
 *          assocId    - pointer to the current association
 *          event      - current event
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *                 A_associate()
 *                 A_release()
 *                 A_abort
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/

int
ACSE(assoc_list, assocId, event)
    struct queue_struct **assoc_list;
    struct assoc_struct **assocId;
    int event;
{
    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ACSE ...\n");
#endif

    if (!(*assocId))
    {

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ACSE: assocId is empty\n");
#endif

        return (FAILED);
    }

    switch (event)
    {

        case A_ASSOCIATE:
            if ((*assocId)->socket_status[CLIENTSOCK] != CS_CONNECTED)
            {
                result = A_associate(assocId);
            }
            else
            {
                result = cs_P_pabort_request(assoc_list, assocId);
            }
            break;

        case A_RELEASE_INDICATION:
            if ((*assocId)->socket_status[CLIENTSOCK] == CS_CONNECTED)
            {
                result = A_release(assoc_list, assocId);
            }
            else
            {
                result = cs_P_pabort_request(assoc_list, assocId);
            }
            break;

        case A_ABORT_INDICATION:
        case AP_ABORT_INDICATION:
        case IR_ABORT_REQUEST:
            result = cs_P_pabort_request(assoc_list, assocId);
            break;

        default:
            result = FAILED;

    }                                /* end of switch */

    return (result);

}                                /* End of Procedure ACSE */

/*****************************************************************************
 *
 * Function: A_associate
 *
 * Purpose: This function connects and gets the new socketFd from the client
 *          and updates the current association IDs: socket_no[CLIENTSOCK] and
 *          socket_status[CLIENTSOCK]=CS_CONNECTED.
 *
 * Input Arguments:
 *          assocId    - pointer to the current association
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              cs_Pconnect_request()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/

int
A_associate(assocId)
    struct assoc_struct **assocId;
{
    int new_socket_no, result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling A_associate ...\n");
#endif

    if ((*assocId) != NULL)
    {
        if ((*assocId)->socket_status[CLIENTSOCK] == CS_IDLE)
        {
            result = cs_Pconnect_request((*assocId)->socket_no[CLIENTSOCK], (struct sockaddr *) &SERVER_ADDRESS, &new_socket_no);

			if (result == FAILED)
			{
				/* When debugging statements are off, if connecting to client 
                /* failed the first time, then slow down for 1 second and
                /* then try again. 
				 */
#if (! (defined(DEBUG_LEVEL1) && defined(DEBUG_LEVEL2) && defined(DEBUG_LEVEL3)))
				sleep(1);
#endif
				return(FAILED);
			}

            (*assocId)->socket_no[CLIENTSOCK] = new_socket_no;

#ifdef DEBUG_LEVEL3
            fprintf(DEBUG_FILE3, "A_associate: result=%d, new_socket_no=%d\n", result, new_socket_no);
#endif

            (*assocId)->socket_status[CLIENTSOCK] = CS_CONNECTED;

            return (SUCCESS);
        }
        else
        {
            return (FAILED);
        }
    }
    else
    {
        return (FAILED);
    }

}                                /* End of Procedure A_associate */

/*****************************************************************************
 *
 * Function: A_release
 *
 * Purpose: This function closes the client and QRM ports and removes
 *          the current association from the association list.
 *
 * Input Arguments:
 *          assoc_list - pointer to the association list
 *          assocId    - pointer to the current association
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *          ir_assoc_delete()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/

int
A_release(assoc_list, assocId)
    struct queue_struct **assoc_list;
    struct assoc_struct **assocId;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling A_release ...\n");
#endif

    if ((*assocId) != NULL)
    {
        if ((*assocId)->socket_status[CLIENTSOCK] == CS_CONNECTED)
        {
            (*assocId)->socket_status[CLIENTSOCK] = CS_IDLE;
            close((*assocId)->socket_no[CLIENTSOCK]);
            close((*assocId)->socket_no[QRMSOCK]);

        }

		/* delete the association entry */
		ir_assoc_delete(assoc_list, assocId);

		return (SUCCESS);
    }
    else
    {
        return (FAILED);
    }

}                                /* End of Procedure A_release */

/*****************************************************************************
 *
 * Function: A_abort
 *
 * Purpose: This function closes the client and QRM ports and removes
 *          the current association from the association list.
 *
 * Input Arguments:
 *          assoc_list - pointer to the association list
 *          assocId    - pointer to the current association
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *          ir_assoc_delete()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/

int
A_abort(assoc_list, assocId)
    struct queue_struct **assoc_list;
    struct assoc_struct **assocId;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling A_abort ...\n");
#endif

    if ((*assocId)->socket_status[CLIENTSOCK] == CS_CONNECTED)
    {

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ACSE: A_abort: close sockets for client and QRM\n");
#endif

        close((*assocId)->socket_no[CLIENTSOCK]);
        close((*assocId)->socket_no[QRMSOCK]);
        (*assocId)->socket_status[CLIENTSOCK] = A_P_ABORTED;

	}

	if (*assocId)
	{
        /* delete the association entry */
        ir_assoc_delete(assoc_list, assocId);
	}

	return (SUCCESS);
}                                /* End of Procedure A_abort */
