
/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: cspliset.c
 *
 * Purpose: This file contains the functions to initialize the TCP socket.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Log: ir.h,v $
 *
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/errno.h>
#include "ir.h"
#include "irconst.h"
#include "irerror.h"

#define MAXNAMELEN       50
#define MaxQueueSize      5

struct sockaddr_in SERVER_ADDRESS;

extern int errno;

/*****************************************************************************
 *
 * Function:    cs_PLI_setup
 *
 * Purpose:     This module sets up the TCP IR service berkerley socket.
 *
 * Input Arguments: irPort - TCP IR Default Service Port
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments: irSocket - IR Socket File Id
 *
 * External Variables:
 *
 * Function used:
 *                  socket()
 *                  display_error()
 *                  setsockopt()
 *                  gethostname()
 *                  gethostbyname()
 *                  htons()
 *                  bind()
 *                  listen()
 *
 * Algorithm: 
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: 
 *
 * Notes:
 *
 ****************************************************************************/
int
cs_PLI_setup(irPort, irSocket)
    int irPort, *irSocket;
{
    struct hostent *hostInfo;
    char host_name[MAXNAMELEN], domain_name[MAXNAMELEN];
    int optval, i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "calling ... cs_PLI_setup \n");
#endif

    /* Open a TCP Socket to communicate with an IR client */
    if ((*irSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
        display_error("KERNEL", ERR_CREATE_SOCKET, errno);
        return (FAILED);
    }

    optval = 1;
    if (setsockopt(*irSocket, SOL_SOCKET, SO_REUSEADDR, (char *) &optval, sizeof(optval)) < 0)
    /*if (setsockopt(*irSocket, SOL_SOCKET, (char *) &optval, sizeof(optval)) < 0)*/
    {
        display_error("KERNEL", ERR_SET_SOCKET_OPTIONS, errno);
        return(FAILED);
    }

    /* Bind our local address so that a client can send to us */
    bzero((char *) &SERVER_ADDRESS, sizeof(struct sockaddr_in));
    SERVER_ADDRESS.sin_family = AF_INET;

    /* Get our hostname */
    if (gethostname(host_name, MAXNAMELEN) < 0)
    {
        display_error("KERNEL", ERR_CANNOT_GET_HOST_NAME, errno);
        return(FAILED);
    }

    if (getdomainname(domain_name, MAXNAMELEN) < 0)
	{
        display_error("KERNEL", ERR_CANNOT_GET_DOMAIN_NAME, errno);
        return(FAILED);
	}

	for (i=0; i<strlen(domain_name); i++)
		if (isalpha(domain_name[i]))
			domain_name[i] = tolower(domain_name[i]);
    strcat(host_name, ".");
    strcat(host_name, domain_name);

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_PLI_setup: host name=%s\n", host_name);
#endif

    /* Get our IP address */
    hostInfo = gethostbyname(host_name);
    if (hostInfo != NULL)
    {
#ifdef VMS
    SERVER_ADDRESS.sin_addr.s_addr = INADDR_ANY;
#else
	SERVER_ADDRESS.sin_addr.s_addr = inet_network(inet_ntoa(*hostInfo->h_addr_list));

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_PLI_setup: s_addr = %lu\n", SERVER_ADDRESS.sin_addr.s_addr);
#endif

#endif
    }
    else
    {
        display_error("KERNEL", ERR_CANNOT_GET_HOST_NAME, errno);
        return(FAILED);
    }
    SERVER_ADDRESS.sin_port = htons(irPort);

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_PLI_setup: sin_port = %d, irport = %d\n", SERVER_ADDRESS.sin_port, irPort);
#endif


    if (bind(*irSocket, (struct sockaddr *) & SERVER_ADDRESS, sizeof(SERVER_ADDRESS)) < 0)
    {
        display_error("KERNEL", ERR_BIND_SOCKET, errno);
        return (FAILED);
    }

    /* listen for connection requests */
    if (listen(*irSocket, MaxQueueSize) < 0)
    {
        display_error("KERNEL", ERR_LISTEN_SOCKET, errno);
        return (FAILED);
    }

    return (SUCCESS);
}                                /* end of ira_ALI_setup  */
