/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: ircomevt.c
 *
 * Purpose: This file contains the function to poll all associations.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used: ira_comm_event()
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Log: ir.h,v $
 *
 *****************************************************************************/

#include <stdio.h>
#include <time.h>
#include "ir.h"
#include "irerror.h"
#include "irconst.h"
#include "irassoc.h"
#include "irstatab.h"
#include "asn_incl.h"

/*****************************************************************************
 *
 * Function: ira_comm_event
 *
 * Purpose: This function poll all the associations and return an event and
 *          a decoded PDU. It also keeps track of the event that requires a
 *          response with a timer; it will release that particular association
 *          when it exceeds the response timeout factor. If a particular
 *          association is idle for too long, it will release that association
 *          automatically.
 *
 * Input Arguments:  serviceSocket       - IR service socket
 *                   assoc_list          - list of associations
 *
 * Return values:    event               - an event from the received PDU
 *
 * Output Arguments: assocId - placed the received APDU and its size in
 *                             APDU_ptr and APDUsize respectively.
 *                   PDU_ptr - the decoded PDU from the received APDU.
 *
 * Function used:    cs_PollAssoc
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ira_comm_event(serviceSocket, assoc_list, assocId, PDU_ptr)
    int serviceSocket;                /* IR Service Socket */
    struct queue_struct **assoc_list;        /* list of associations */
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{
    int event;

#ifdef DEBUG_LEVEL3
    ir_view_assoc_list(assoc_list);
#endif

    (*assocId) = (*assoc_list)->front;

    /* Poll each associations */
    do
    {

        event = cs_PollAssoc(serviceSocket, assoc_list, assocId, PDU_ptr);

#ifdef DEBUG_LEVEL3
        fprintf(DEBUG_FILE3, "ira_comm_event1: event=%d\n", event);
        if ((*assocId))
            fprintf(DEBUG_FILE3, "ira_comm_event2: current assocId->socket=%d\n", (*assocId)->socket_no[CLIENTSOCK]);
#endif

        if ((*assocId) && (event == NON_EVENT))
        {
            /*
             * if the request has not responded in the predetermine timeout,
             * abort
             */
            if (ir_alarm_expires(assocId))
            {
                display_error("KERNEL", ERR_TIMEOUT, -1);
                return (IR_ABORT_REQUEST);
            }

            if (ir_idle_too_long(assocId))
            {
                display_error("KERNEL", ERR_IDLE_TIMEOUT, -1);
                return (IR_ABORT_REQUEST);
            }

            (*assocId) = (*assocId)->next;
        }

#ifdef DEBUG_LEVEL3
        if ((*assocId))
            fprintf(DEBUG_FILE3, "ira_comm_event3: next assocId=%d\n", (*assocId)->socket_no[CLIENTSOCK]);
#endif

    } while ((*assocId) && (event == NON_EVENT));

    /* Update the current time for checking the timeout in the case of */
    /* the client drops the line. */
    if (event != NON_EVENT)
        (*assocId)->last_event_time = time(NULL);

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ira_comm_event4: event=%d\n", event);
#endif

    return (event);

}                                /* end of procedure ira_comm_event */
