/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irspawnp.c
 *
 * Purpose: This file contains functions to start the init target process and
 *          connect between the IR kernel and the target process.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used: ir_spawn_process()
 *                 ira_link_to_init()
 *                 ira_invoke_target_process()
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Log: ir.h,v $
 *
 *****************************************************************************/

#include <stdio.h>
#include <time.h>
#include <rpc/rpc.h>
#include <sys/errno.h>
#include "ir.h"
#include "tpstartu.h"
#include "irerror.h"
#include "irassoc.h"
#include "irconst.h"

extern int errno;

/*****************************************************************************
 *
 * Function: ir_spawn_process
 *
 * Purpose: This function creates a RPC handler and then does a RPC call
 *          to the target process. The called RPC function will return a TCP
 *          port number via name pipe so that the IR kernel can connect to
 *          target process using the right TCP port number.
 *
 * Input Arguments:
 *          target_host_name - target process host name
 *
 * Return values: TCP port number for connecting to target process that is
 *                returned from the init target process by name pipe.
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *                  clnt_create()
 *                  display_error()
 *                  tp_startup_1()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_spawn_process(target_host_name)
    char *target_host_name;
{

    int *TCP_port_no;
    CLIENT *cl;
    int *result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_spawn_process\n");
#endif

    /* Create the client "handle". */
    if ((cl = clnt_create(target_host_name, TP_STARTUP_PROGRAM, TP_STARTUP_VERS, "tcp")) == NULL)
    {
        display_error("KERNEL", ERR_CREATE_TCP_HANDLER, -1);
        return (0);
    }

    if ((TCP_port_no = tp_startup_1(NULL, cl)) == NULL)
    {
        display_error("KERNEL", ERR_RPC_CALL_FAIL, errno);
        return (0);
    }

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_spawn_process: port=%d\n", *TCP_port_no);
#endif

    clnt_destroy(cl);                /* done with handler */

    return (*TCP_port_no);
}

/*****************************************************************************
 *
 * Function: ira_link_to_init
 *
 * Purpose: This function connects to the target process and updates the
 *          current association's Id socket_no[QRMSOCK] to the new socket Fd.
 *
 * Input Arguments:
 *                  TCP_port_no - TCP port number to talk to target process
 *                  host_name - target process host name
 *
 * Return values:   SUCCESS - connect to target process successfully
 *                  FAILED  - failed to connect to target process
 *
 * Output Arguments:
 *                  assocId - current association
 *
 * External Variables:
 *
 * Function used:
 *                  cs_connect()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ira_link_to_init(TCP_port_no, host_name, assocId)
    int TCP_port_no;
    char *host_name;
    struct assoc_struct **assocId;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ira_link_to_init\n");
#endif

    /* Connect to the init process and update the QRM socket no */
    return (cs_connect(TCP_port_no, host_name, assocId));
}

/*****************************************************************************
 *
 * Function: ira_invoke_target_process
 *
 * Purpose: This function initiate the init target process by calling
 *          ir_spawn_process() and ira_link_to_init().
 *
 * Input Arguments:
 *          host_name - target process host name
 *          assocId   - current association
 *
 * Return values:   SUCCESS - create and connect to the init target process
 *                            successfully
 *                  FAILED  - failed to create and connect to init target process
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Functions used: ir_spawn_process()
 *                 ira_link_to_init()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ira_invoke_target_process(host_name, assocId)
    char *host_name;
    struct assoc_struct **assocId;
{
    int TCP_port_no;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ira_invoke_target_process\n");
#endif

    TCP_port_no = ir_spawn_process(host_name);
    if (TCP_port_no == 0)
        return (FAILED);
    else
    {
        return (ira_link_to_init(TCP_port_no, host_name, assocId));
    }
}
