/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tprsrpt.c
 *
 * Purpose: This is the target process black box resource control report
 *          program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Log: ir.h,v $
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "ir_oid.h"
#include "asn_incl.h"
#include "irerror.h"
#include "tpconst.h"
#include "tpbbox.h"

AsnInt RSRPT_REFERENCE_ID_LENGTH;
char *RSRPT_REFERENCE_ID;

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_resourceReport_response
 *
 * Purpose: This function fills and creates the resource report response PDU.
 *          The DB record is filled with the data file 'canmarc'.
 *
 * Input Arguments:
 *          report_result -   0 = Access control failed
 *                            3 = Resource control failed
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_ptr - resourceReport response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *              ASN_MakeOid
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_resourceReport_response(report_result, PDU_ptr)
    int report_result;
    APDU *PDU_ptr;
{

    NamePlusRecord *data_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_resourceReport_response\n");
#endif

    PDU_ptr->a.resourceReportResp = (struct ResourceReportResp *) malloc(sizeof(struct ResourceReportResp));

    PDU_ptr->choiceId = APDU_RESOURCEREPORTRESP;

    PDU_ptr->a.resourceReportResp->referenceId.octetLen = RSRPT_REFERENCE_ID_LENGTH;
    PDU_ptr->a.resourceReportResp->referenceId.octs = (char *) malloc(RSRPT_REFERENCE_ID_LENGTH);
    memcpy(PDU_ptr->a.resourceReportResp->referenceId.octs, RSRPT_REFERENCE_ID, RSRPT_REFERENCE_ID_LENGTH);

    PDU_ptr->a.resourceReportResp->resourceReportStatus = report_result;

    PDU_ptr->a.resourceReportResp->resourceReport = (EXTERNAL *) malloc(sizeof(EXTERNAL));

    ASN_MakeOid((char *) OID_CLASS_RESOURCE_REPORT_FORMAT, &PDU_ptr->a.resourceReportResp->resourceReport->direct_reference);

    PDU_ptr->a.resourceReportResp->resourceReport->indirect_reference = (AsnInt *) malloc(sizeof(AsnInt));
    *PDU_ptr->a.resourceReportResp->resourceReport->indirect_reference = 0;

    PDU_ptr->a.resourceReportResp->resourceReport->data_value_descriptor.octetLen = 0;
    PDU_ptr->a.resourceReportResp->resourceReport->data_value_descriptor.octs = NULL;

    PDU_ptr->a.resourceReportResp->resourceReport->encoding = (struct EXTERNALChoice *) malloc(sizeof(struct EXTERNALChoice));
    PDU_ptr->a.resourceReportResp->resourceReport->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
    PDU_ptr->a.resourceReportResp->resourceReport->encoding->a.octet_aligned = (AsnOcts *) malloc(sizeof(AsnOcts));
    PDU_ptr->a.resourceReportResp->resourceReport->encoding->a.octet_aligned->octetLen = sizeof("RESOURCE REPORT") - 1;
    PDU_ptr->a.resourceReportResp->resourceReport->encoding->a.octet_aligned->octs = (char *) malloc(sizeof("RESOURCE REPORT") - 1);
    memcpy(PDU_ptr->a.resourceReportResp->resourceReport->encoding->a.octet_aligned->octs, "RESOURCE REPORT", PDU_ptr->a.resourceReportResp->resourceReport->encoding->a.octet_aligned->octetLen);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_create_resourceReport_response
 *
 * Purpose: This function creates the resource report response PDU, encodes,
 *          and creates the resource report response APDU for sending to
 *          IR kernel.
 *
 * Input Arguments:
 *          Report_result -   0 = No error (DB record)
 *                            8 = Mandatory fields are invalid (Diag record)
 *                           31 = Resource control failed (Diag record)
 *                          101 = Access control failed (Diag record)
 *          data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_resourceReport_response()
 *              InitNibbleMem()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *              FreeSearchResp()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_resourceReport_response(report_result, outputBuf, data_ptr, APDU_length)
    int report_result;
    SBuf *outputBuf;
    char **data_ptr;
    int *APDU_length;
{
    APDU PDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_resourceReport_response()\n");
#endif

    tp_fill_resourceReport_response(report_result, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX: tp_create_resourceReport_response, begin dump data:\n");
    PrintResourceReportResp(DEBUG_FILE2, PDU_ptr.a.resourceReportResp, 0);
    fprintf(DEBUG_FILE2, "BBOX: tp_create_resourceReport_response, end dump data:\n");
#endif

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:Resource Report Response APDU Dump from tprsrpt\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    FreeResourceReportResp(PDU_ptr.a.resourceReportResp);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_resourceReport_request
 *
 * Purpose: This function performs the following functions:
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the resourceReport response APDU to
 *                IR kernel.
 *
 * Input Arguments:
 *              PDU - resourceReport request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              FreeResourceReportReq()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_resourceReport_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_resourceReport_request(PDU, data_ptr, socketFd)
    APDU *PDU;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_resourceReport_request\n");
#endif

    /* save reference ID */
    RSRPT_REFERENCE_ID_LENGTH = PDU->a.resourceReportReq->referenceId.octetLen;
    RSRPT_REFERENCE_ID = (char *) malloc(PDU->a.resourceReportReq->referenceId.octetLen);
    memcpy(RSRPT_REFERENCE_ID, PDU->a.resourceReportReq->referenceId.octs, PDU->a.resourceReportReq->referenceId.octetLen);

    FreeResourceReportReq(PDU->a.resourceReportReq);

    if (tp_access_control(socketFd, data_ptr, RSRPT_REFERENCE_ID_LENGTH, RSRPT_REFERENCE_ID) == FAILED)
    {
        display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
        tp_create_resourceReport_response(3, &APDU_ptr, data_ptr, &APDU_length);
    }
    else
        tp_create_resourceReport_response(0, &APDU_ptr, data_ptr, &APDU_length);

    if (tpa_send_APDU(socketFd, &APDU_ptr, (int) APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

#ifdef DEBUG_LEVEL1
    else
        fprintf(DEBUG_FILE1, "BBOX: tp_process_resourceReport_request: %ld resourceReport response sent\n", APDU_length);
#endif

    return (SUCCESS);
}
